package com.efaxdeveloper.util.inbound;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;

/**
 * <p>
 * <b>InboundRequestServlet (Sample code)</b>
 * </p>
 * <p>
 * <b>Overview</b>
 * </p>
 * <blockquote>
 * <p>
 * This servlet class is provided as sample code for eFax Developer&trade;
 * inbound POST request processing.
 * </p>
 * </blockquote>
 */
public class InboundRequestServlet extends HttpServlet {

  private static final long   serialVersionUID = 2005670482517353340L;
  private static final String CONTENT_TYPE     = "text/html";
  private static final String TEST_DIRECTORY   = "C:\\samples\\inbound\\";
  
  /** Initialize global variables */
  public void init(ServletConfig config) throws ServletException {
    super.init(config);
  }
  
  public void doGet(HttpServletRequest request, HttpServletResponse response)
      throws ServletException, IOException {
    
    response.setContentType(CONTENT_TYPE);
    PrintWriter out = response.getWriter();
    
    // Begin building the HTML response
    out.println("<html>");
    out.println("<head><title>InboundRequestServlet</title></head>");
    out.println("<body bgcolor=\"#ffffff\">");
    out.println("<p>The servlet has received a GET. This is the reply.</p>");
    
    try {
      // Establish a File object pointing to the XML data file (primarily used
      // for testing)
      File xmlFile = new File(new StringBuffer(TEST_DIRECTORY).append("Sample.xml").toString());
      // Establish the path to your XSD schema when XML validation is desired
      String xsdPath = new StringBuffer(TEST_DIRECTORY).append("Sample.xsd").toString();
      // Instantiate the InboundRequest object by passing a string or File object
      // containing the XML data content. For the sake of testing, InboundRequest
      // accepts a File object, while in reality, the "xml" parameter passed by
      // eFax Developer XML POST will contain the XML data string to be processed.
      // When an xsdPath is passed to any constructor, the XML string will be validated.
      InboundRequest req = new InboundRequest(xmlFile, xsdPath);
      // When the client account does not return page splits (most likely)
      if (!req.isPageSplitting()) {
        // Decode the Base64 encoded document to a specified location. The file
        // will be stored to your specified path using the fax name retrieved from
        // the XML POST FaxName parameter. Note that you are providing a directory
        // path to a location where you would like the physical file to be written.
        // The following method call is not required. If the following method is not
        // called, a hard copy of the physical fax will not be generated on your server.
        req.getDocumentAsFile(TEST_DIRECTORY);
      }
      // Otherwise, the client account is set to return page splits
      else {
        // Write each page to a specified file path
        req.getPagesAsFiles(TEST_DIRECTORY);
      }
      
      out.println("<br>UserName:      " + req.getUserName());
      out.println("<br>Password:      " + req.getPassword());
      out.println("<br>RequestDate:   " + req.getRequestDateAsString());
      out.println("<br>RequestType:   " + req.getRequestType());
      out.println("<br>AccountID:     " + req.getAccountID());
      out.println("<br>NumberDialed:  " + req.getNumberDialed());
      out.println("<br>DateReceived:  " + req.getDateReceivedAsString());
      out.println("<br>FaxName:       " + req.getFaxName());
      out.println("<br>FileType:      " + req.getFileType());
      out.println("<br>PageCount:     " + req.getPageCount());
      out.println("<br>CSID:          " + req.getCSID());
      out.println("<br>ANI:           " + req.getANI());
      out.println("<br>Status:        " + req.getStatus());
      out.println("<br>MCFID:         " + req.getMCFID());
      
      // When user field name/value pairs have been passed (optional setting)
      if (req.hasUserFields()) {
        // Begin processing user name/value pairs
        out.println("<p><b>UserField name/value pair(s) follow:<br></b>");
        // Establish a TreeMap object to hold the user name/value pairs
        Map<String, String> userFields = new TreeMap<String, String>(req.getUserFields());
        // Store the TreeMap contents to an ArrayList object for sorting
        ArrayList<String> keys = new ArrayList<String>(userFields.keySet());
        // Establish an Iterator object for the List object
        Iterator<String> iterator = keys.iterator();
        // Establish a String object to hold the user field name
        String fieldName = "";
        // Establish a String object to hold the user field value
        String fieldValue = "";
        // Spin through the list of user name/value pairs
        while (iterator.hasNext()) {
          // Store the field name value
          fieldName = iterator.next();
          // Store the field value
          fieldValue = userFields.get(fieldName);
          // Print the user name/value pairs received in the XML
          out.println("<br>" + fieldName + ": " + fieldValue);
        }
        out.println("</p>");
      }
      
      // When barcode fields have been passed (optional setting)
      if (req.hasBarcodes()) {
        // Begin processing barcode fields
        out.println("<p><b>Barcode value(s) follow:</b><br>");
        // Establish a null Barcode reference
        Barcode b = null;
        // Establish a new ArrayList object to hold all Barcode objects. Use
        // the getBarcodesByPage(int) method to return all Barcode objects
        // from a specified page.
        ArrayList<Barcode> barcodes = new ArrayList<Barcode>(req.getBarcodes());
        // Establish an iterator Object
        Iterator<Barcode> iterator = barcodes.iterator();
        // Process each Barcode object
        while (iterator.hasNext()) {
          // Retrieve a Barcode object
          b = new Barcode(iterator.next());
          // Retrieve the Barcode object state
          out.println("<br>Key:           " + b.getKey());
          out.println("<br>ReadSequence:  " + b.getReadSequence());
          out.println("<br>ReadDirection: " + b.getReadDirection());
          out.println("<br>Symbology:     " + b.getSymbology());
          out.println("<br>PageNumber:    " + b.getPageNumber());
          out.println("<br>XStartPointA:  " + b.getXStartPointA());
          out.println("<br>XStartPointB:  " + b.getXStartPointB());
          out.println("<br>YStartPointA:  " + b.getYStartPointA());
          out.println("<br>YStartPointB:  " + b.getYStartPointB());
          out.println("<br>XEndPointA:    " + b.getXEndPointA());
          out.println("<br>XEndPointB:    " + b.getXEndPointB());
          out.println("<br>YEndPointA:    " + b.getYEndPointA());
          out.println("<br>YEndPointB:    " + b.getYEndPointB());
          out.println("<br>--------------------------------------------");
        }
        out.println("</p>");
      }
      
      // The existence of the following string anywhere in your output display
      // will cause eFax Developer to move the inbound fax into your desired
      // "Good" folder. Otherwise, eFax Developer will consider the XML POST
      // attempt a failure and the inbound fax will be moved into your desired
      // "Bad" folder.
      out.println("<p>Post Successful</p>");
      
    }
    catch (Exception e) {
      e.printStackTrace();
      out.println("<p>Post Failed!</p>");
    }
    catch (Error err) {
      err.printStackTrace();
      out.println("<p>Post Failed!</p>");
    }
    finally {
      out.println("</body></html>");
      out.flush();
      out.close();
    }
    
  }
  
  public void destroy() {
    
  }
  
}