package com.efaxdeveloper.util.outbound;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * <p><b>OutboundRequestServlet (Sample code)</b></p>
 * <p><b>Overview</b></p>
 * <blockquote>
 * <p>This servlet class is provided as sample code for outbound request processing.</p>
 * </blockquote>
 */
public class OutboundRequestServlet extends HttpServlet {
  private static final long serialVersionUID = 7007096364583401713L;
  private static final String CONTENT_TYPE = "text/html";

  /**Initialize global variables*/
  public void init(ServletConfig config) throws ServletException {
    super.init(config);
  }

  public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

    response.setContentType(CONTENT_TYPE);
    PrintWriter out = response.getWriter();

    // Instantiate a new OutboundRequest object
    OutboundRequest req = new OutboundRequest();

/*----------------------------------------------------------------*/
/*----------------- Set response type ----------------------------*/
/*----------------------------------------------------------------*/
    // Set your desired response type (XML response is default)
    req.setHTMLResponse(false);

/*----------------------------------------------------------------*/
/*----------------- Set user identification ----------------------*/
/*----------------------------------------------------------------*/
    // Set your eFax Developer outbound account identifier (required)
    req.setAccountID("");
    // Set your eFax Developer outbound user name (required)
    req.setUserName("");
    // Set your eFax Developer outbound password (required)
    req.setPassword("");

    try {
      // Build your outbound transmission request
      buildRequest(req);
      // POST the outbound transmission request to eFax Developer
      OutboundResponse resp = req.sendFax();
      // When an XML response is selected (default)
      if (!resp.isHTMLResponse()) {
        // Process the XML response
        processXMLResponse(resp, out);
      }
      // Otherwise, an HTML response is desired
      else {
        // Process the HTML response
        processHTMLResponse(resp, out);
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    catch (Error err) {
      err.printStackTrace();
    }
    finally {
      out.flush();
      out.close();
    }

  }

  private void buildRequest(OutboundRequest req) throws Exception, IOException {

    /*---------------------------------------------------------------*/
    /*---------------- Set transmission control ---------------------*/
    /*---------------------------------------------------------------*/
    // Set a unique transmission identifier (recommended)
    req.setTransmissionID("");
    // Set a client specified customer identifier when desired (optional)
    req.setCustomerID("");
    // Set the transmission identifier validation to fail duplicates (if desired)
    req.setNoDuplicates(false);
    // Set your desired resolution ("Standard mode" is default)
    req.setFineResolution(false);
    // Set your desired priority ("Normal" is default)
    req.setHighPriority(false);
    // Set your desired "self-busy" option ("Enable" is default)
    req.setSelfBusy(true);
    // Set your desired "Dynamic Fax Header" (optional)
    req.setFaxHeader("");
    // Override the account's TSID ("Your assigned fax number transmits as the TSID by default")
    req.setTSID("");

    /*---------------------------------------------------------------*/
    /*---------------- Set optional disposition control -------------*/
    /*---------------------------------------------------------------*/
    // Set your desired disposition language (English is default)
    req.setDispositionLanguage(OutboundRequest.ISO_ENGLISH);
    // Set your desired disposition level ("ERROR," "SUCCESS," "BOTH," or "NONE" <-- default)
    req.setDispositionLevel(OutboundRequest.DL_BOTH);
    // Set final disposition notifications via HTTP POST to a designated URL (if desired)
    req.setDispositionsTo("https://secure.efaxdeveloper.com/postok.html");

    // Instantiate an EmailBundler object    
    EmailBundler dispositionEmails = new EmailBundler();

    // Add new recipient(s) to the EmailBundler object
    dispositionEmails.add("");
//  dispositionEmails.add("add another");
//  dispositionEmails.add("and another");
//  dispositionEmails.add("for as many disposition email addresses as you desire");

    // Set final disposition notifications via email (overrides previous setDispositionsTo())
    req.setDispositionsTo(dispositionEmails);

    /*---------------------------------------------------------------*/
    /*---------------- Set recipient information --------------------*/
    /*---------------------------------------------------------------*/
    // Set the recipient name for this transmission (optional)
    req.setRecipientName("");
    // Set the recipient company name for this transmission (optional)
    req.setRecipientCompany("");
    // Set the recipient fax number for this transmission (required)
    req.setRecipientFax("");

    /*---------------------------------------------------------------*/
    /*---------------- Set files to transmit ------------------------*/
    /*---------------------------------------------------------------*/
    // Instantiate a DocumentBundler object
    DocumentBundler docs = new DocumentBundler();

    // Add document(s) to the DocumentBundler object by path
    docs.add("C:\\your\\document\\path\\your.doc");
    // Add a document as a string if you desire to the DocumentBundler object
    docs.add("<html><head></head><title>This is a test</title><body>Here is a test HTML string</body></html>", "htm");
    // Another example of adding a document as a string to the DocumentBundler object
    docs.add("This is a test string!", ".txt");
    // Add binary large object (BLOB) data as a byte array. The following line
    // is provided for testing and is an unrealistic example of what will occur
    // in the real world. Realistically, BLOB data could be retrieved from a
    // client database as a byte array. The byte array will be passed to the
    // DocumentBundler(byte[], String) setter method.
    docs.add(getBLOB("C:\\your\\document\\path\\your.xls"), ".xls");

    // Set the documents to be faxed for this transmission (required)
    req.setDocuments(docs);

  }

  /**
   * The following method was added to test binary large object (BLOB) data. This
   * method will generate a byte array that can be passed to the DocumentBundler
   * object as desired for testing purposes.
   */
  private byte[] getBLOB(String documentPath) throws IOException {

    // Establish a new File object pointing to the desired document
    File file = new File(documentPath);
    // Establish a new DataInputStream to the file
    DataInputStream dis = new DataInputStream(new FileInputStream(file));
    // Establish a byte array to hold the byte contents of the file
    byte bytes[] = new byte[dis.available()];
    // Read the DataInputStream completely into the byte array
    dis.readFully(bytes);
    // Close the DataInputStream
    dis.close();

    return bytes;

  }

  private void processHTMLResponse(OutboundResponse resp, PrintWriter out) {

    // Retrieve the raw response
    out.println(resp.getRawResponse());

  }

  private void processXMLResponse(OutboundResponse resp, PrintWriter out) {

    out.println("<html>");
    out.println("<head><title>OutboundRequestServlet</title></head>");
    out.println("<body>");

    // When eFax Developer approved the outbound request
    if (resp.isApproved()) {
      out.println("<p>eFax Developer&trade; successfully received your request</p>");
      out.println("<blockquote>");
      // When a client transmission identifier is provided
      if ((resp.getTransmissionID() != null) && (resp.getTransmissionID().length() > 0)) {
        out.println("Transmission ID: "        + resp.getTransmissionID());
        out.println("<br>");
      }
      out.println("<br>DOC Identifier: "       + resp.getDocID());
      out.println("</blockquote>");
    }
    // Otherwise, eFax Developer rejected the outbound request
    else {
      out.println("<p>eFax Developer&trade; received your request but encountered an error</p>");
      out.println("<blockquote>");
      // When a client transmission identifier is provided
      if ((resp.getTransmissionID() != null) && (resp.getTransmissionID().length() > 0)) {
        out.println("Transmission ID: "        + resp.getTransmissionID());
        out.println("<br>");
      }
      out.println("<br>Error Level: "          + resp.getErrorLevel());
      out.println("<br>Error Message: "        + resp.getErrorMessage());
      out.println("</blockquote>");
    }

    out.println("</body></html>");

  }

  public void destroy() {
  }

}